class PluginPuppetBaselineMakeValueColumnBigger < ActiveRecord::Migration
  def self.up
    change_column :baseline_properties, :value, :text
    change_column :baseline_resources,  :title, :text

    # No more than 4 machine pages per name, total, once we take into account
    # the length field and the UTF8 expansion of char->byte storage.  Also,
    # not eating *too* much of our 64K column size limit.
    change_column :baseline_differences, :resource_name, 'VARCHAR(5120)'

    # ...and populate the columns again, because we might have truncated them
    # before, and it would be suboptimal to leave the damage data when we can
    # recreate it here without too much overhead.
    time = Benchmark.measure do
      STDOUT.puts "-- repopulating resource_name column due to potential truncation"
      BaselineDifference.transaction do
        BaselineDifference.update_all :resource_name => nil

        BaselineDifference.connection.execute(<<SQL)
UPDATE
    baseline_differences, resource_statuses
SET
    baseline_differences.resource_name =
        CONCAT(resource_statuses.resource_type, '[', resource_statuses.title, ']')
WHERE
    resource_statuses.id = baseline_differences.report_resource_id
SQL

        BaselineDifference.connection.execute(<<SQL)
UPDATE
    baseline_differences, baseline_resources
SET
    baseline_differences.resource_name =
        CONCAT(baseline_resources.resource_type, '[', baseline_resources.title, ']')
WHERE
    baseline_resources.id = baseline_differences.baseline_resource_id
    AND baseline_differences.resource_name IS NULL
SQL
      end
    end

    STDOUT.puts "  -> %.4fs" % time.real
  end

  def self.down
    change_column :baseline_resources,  :title, :string
    change_column :baseline_properties, :value, :string

    # This will do whatever truncation is required.
    change_column :baseline_differences, :resource_name, :string, :limit => 255
  end
end
