require 'ostruct'
class ComplianceController < ApplicationController
  unloadable

  def index
    @date = Date.parse(params[:date] || Time.zone.now.to_s)
    @node_summary = build_node_summary

    @other_days = BaselineUtil.other_day_summaries(@date)

    @unreviewed_changes_on_other_days = BaselineDifference.self_comparisons.for_dates_other_than(@date).undecided.count
  end

  private
  def build_node_summary
    OpenStruct.new.tap do |s|
      s.groups = NodeGroup.all(:order => 'name').map do |group|
        group_node_count                  = NodeBaseline.node_count_for_group_and_date(group, @date)
        group_unreviewed_resources        = NodeBaseline.resource_counts_for_group_and_date(group, @date, nil)
        group_accepted_resources          = NodeBaseline.resource_counts_for_group_and_date(group, @date, true)
        group_rejected_resources          = NodeBaseline.resource_counts_for_group_and_date(group, @date, false)

        group_reports = Report.all(
          :group      => 'node_id HAVING MAX(time)',
          :conditions => {
            :node_id => group.node_ids,
            :kind    => 'inspect',
            :time    => (@date...@date.tomorrow),
          }
        )
        group_failed_resources = group_reports.sum do |r|
          r.resource_statuses.count(:conditions => { :failed => true })
        end

        OpenStruct.new(
          :group_id                  => group.id,
          :name                      => group.name,
          :node_count                => group_node_count,
          :failed_resource_count     => group_failed_resources,
          :unreviewed_resource_count => group_unreviewed_resources,
          :accepted_resource_count   => group_accepted_resources,
          :rejected_resource_count   => group_rejected_resources
        )
      end

      s.groups.delete_if do |g|
        g.unreviewed_resource_count + g.accepted_resource_count + g.rejected_resource_count == 0
      end

      node_ids = NodeGroupMembership.find(:all, :select => :node_id, :group => :node_id).map(&:node_id)
      node_ids = [ 0 ] if node_ids.empty?
      s.nodes = Node.find(:all, :conditions => ['id NOT IN(?)', node_ids], :order => 'name').map do |node|
        node_unreviewed_resources        = NodeBaseline.resource_counts_for_node_ids_and_date(node.id, @date, nil)
        node_accepted_resources          = NodeBaseline.resource_counts_for_node_ids_and_date(node.id, @date, true)
        node_rejected_resources          = NodeBaseline.resource_counts_for_node_ids_and_date(node.id, @date, false)

        reports = node.reports.all(
          :group      => 'node_id HAVING MAX(time)',
          :conditions => {
            :kind    => 'inspect',
            :time    => (@date...@date.tomorrow),
          }
        )
        node_failed_resources = reports.sum do |r|
          r.resource_statuses.count(:conditions => { :failed => true })
        end

        OpenStruct.new(
          :node_id                   => node.id,
          :name                      => node.name,
          :failed_resource_count     => node_failed_resources,
          :unreviewed_resource_count => node_unreviewed_resources,
          :accepted_resource_count   => node_accepted_resources,
          :rejected_resource_count   => node_rejected_resources
        )
      end

      s.nodes.delete_if do |n|
        n.unreviewed_resource_count + n.accepted_resource_count + n.rejected_resource_count == 0
      end

      s.custom_reports = CustomReport.all(:order => 'created_at DESC')

      s.node_count                      = s.groups.sum(&:node_count)
      s.group_failed_resource_count     = s.groups.sum(&:failed_resource_count)
      s.group_unreviewed_resource_count = s.groups.sum(&:unreviewed_resource_count)
      s.group_accepted_resource_count   = s.groups.sum(&:accepted_resource_count)
      s.group_rejected_resource_count   = s.groups.sum(&:rejected_resource_count)
      s.node_failed_resource_count      = s.nodes.sum(&:failed_resource_count)
      s.node_unreviewed_resource_count  = s.nodes.sum(&:unreviewed_resource_count)
      s.node_accepted_resource_count    = s.nodes.sum(&:accepted_resource_count)
      s.node_rejected_resource_count    = s.nodes.sum(&:rejected_resource_count)
    end
  end
end
